from .fulfillment_automation import FulfillmentAutomation
from .fulfillment import FulfillmentResource
from .template import TemplateResource


__all__ = [
    'FulfillmentAutomation',
    'FulfillmentResource',
    'TemplateResource'
]



# -*- coding: utf-8 -*-

import json

from connect.config import Config
from connect.logger import function_log
from connect.models import FulfillmentScheme, Param
from .base import BaseResource
from .template import TemplateResource
from .utils import joinurl


class FulfillmentResource(BaseResource):
    resource = 'requests'
    limit = 1000
    scheme = FulfillmentScheme()

    def build_filter(self):
        filters = super(FulfillmentResource, self).build_filter()
        if Config.products:
            filters['product_id'] = Config.products

        filters['status'] = 'pending'
        return filters

    @function_log
    def approve(self, pk, data):
        url = joinurl(self._obj_url(pk), 'approve/')
        return self.api.post(url=url, data=json.dumps(data if data else {}))

    @function_log
    def inquire(self, pk):
        return self.api.post(url=joinurl(self._obj_url(pk), 'inquire/'), data=json.dumps({}))

    @function_log
    def fail(self, pk, reason):
        url = joinurl(self._obj_url(pk), 'fail/')
        return self.api.post(url=url, data=json.dumps({'reason': reason}))

    @function_log
    def render_template(self, pk, template_id):
        return TemplateResource().render(template_id, pk)

    @function_log
    def update_parameters(self, pk, params):
        list_dict = []
        for _ in params:
            list_dict.append(_.__dict__ if isinstance(_, Param) else _)

        return self.api.put(
            url=self._obj_url(pk),
            data=json.dumps({'asset': {'params': list_dict}}),
        )



# -*- coding: utf-8 -*-

import requests

from connect.config import Config
from connect.logger import function_log, logger
from connect.models import BaseScheme, ServerErrorScheme
from connect.models.exception import ServerErrorException
from .utils import joinurl

config = Config()


class ApiClient(object):

    @property
    def headers(self):
        config.check_credentials(
            config.api_url, config.api_key, config.products)
        return {
            "Authorization": config.api_key,
            "Content-Type": "application/json",
        }

    @staticmethod
    def check_response(response):
        if not hasattr(response, 'content'):
            raise AttributeError(
                'Response not attribute content. Check your request params'
                'Response status - {}'.format(getattr(response, 'code')),
            )

        if not hasattr(response, 'ok') or not response.ok:
            data, error = ServerErrorScheme().loads(response.content)
            if data:
                raise ServerErrorException(data)

        return response.content

    @function_log
    def get(self, url, params=None, **kwargs):
        kwargs['headers'] = self.headers
        response = requests.get(url, params, **kwargs)
        return self.check_response(response)

    @function_log
    def post(self, url, data=None, json=None, **kwargs):
        kwargs['headers'] = self.headers
        response = requests.post(url, data, json, **kwargs)
        return self.check_response(response)

    @function_log
    def put(self, url, data=None, **kwargs):
        kwargs['headers'] = self.headers
        response = requests.put(url, data, **kwargs)
        return self.check_response(response)


class BaseResource(object):
    resource = None
    limit = 100
    api = ApiClient()
    scheme = BaseScheme()

    def __init__(self, *args, **kwargs):

        if self.__class__.resource is None:
            raise AttributeError('Resource name not specified in class {}'.format(
                self.__class__.__name__) + '. Add an attribute `resource` name of the resource')

    def build_filter(self):
        res_filter = {}
        if self.limit:
            res_filter['limit'] = self.limit

        return res_filter

    @property
    def _list_url(self):
        return joinurl(config.api_url, self.__class__.resource)

    def _obj_url(self, pk):
        return joinurl(self._list_url, pk)

    def __loads_scheme(self, response):
        objects, error = self.scheme.loads(response, many=True)
        if error:
            raise TypeError(
                'Invalid structure for initialisation objects. \n'
                'Error: {}. \nServer Response: {}'.format(error, response),
            )

        return objects

    def get(self, pk):
        response = self.api.get(url=self._obj_url(pk))
        objects = self.__loads_scheme(response)
        if isinstance(objects, list) and len(objects) > 0:
            return objects[0]

    def list(self):
        filters = self.build_filter()
        logger.info('Get list request by filter - {}'.format(filters))
        response = self.api.get(url=self._list_url, params=filters)
        return self.__loads_scheme(response)



from marshmallow import Schema, fields, post_load


class BaseObject:
    def __init__(self, *args, **kwargs):
        self.id = kwargs.get('id')
        if kwargs:
            for attr, val in kwargs.items():
                setattr(self, attr, val)


class BaseScheme(Schema):
    id = fields.Str()

    @post_load
    def make_object(self, data):
        return BaseObject(**data)


class Product(BaseObject):
    pass


class ProductScheme(BaseScheme):
    name = fields.Str()

    @post_load
    def make_object(self, data):
        return Product(**data)


class Item(BaseObject):
    pass


class ItemScheme(BaseScheme):
    global_id = fields.Str()
    mpn = fields.Str()
    old_quantity = fields.Str()
    quantity = fields.Integer()

    @post_load
    def make_object(self, data):
        return Item(**data)


class Tier(BaseObject):
    pass


class TierScheme(BaseScheme):
    name = fields.Str()
    contact_info = fields.Dict()
    external_id = fields.Str()
    external_uid = fields.UUID()

    @post_load
    def make_object(self, data):
        return Tier(**data)


class TiersSchemeMixin(Schema):
    customer = fields.Nested(TierScheme)
    tier1 = fields.Nested(TierScheme)
    tier2 = fields.Nested(TierScheme)


class Hub(BaseObject):
    pass


class HubScheme(BaseScheme):
    name = fields.Str()

    @post_load
    def make_object(self, data):
        return Hub(**data)


class HubsSchemeMixin(Schema):
    hub = fields.Nested(HubScheme, only=('id', 'name'))
    external_id = fields.Str()


class Company(BaseObject):
    pass


class CompanyScheme(BaseScheme):
    name = fields.Str()

    @post_load
    def make_object(self, data):
        return Company(**data)


class Connection(BaseObject):
    pass


class ConnectionScheme(BaseScheme):
    type = fields.Str()
    provider = fields.Nested(CompanyScheme, only=('id', 'name'))
    vendor = fields.Nested(CompanyScheme, only=('id', 'name'))
    product = fields.Nested(ProductScheme)
    hub = fields.Nested(HubScheme)

    @post_load
    def make_object(self, data):
        return Connection(**data)


class ValueChoice(BaseObject):
    pass


class ValueChoiceScheme(Schema):
    value = fields.Str()
    label = fields.Str()

    @post_load
    def make_object(self, data):
        return ValueChoice(**data)


class Param(BaseObject):
    pass


class ParamsScheme(BaseScheme):
    name = fields.Str()
    type = fields.Str()
    value = fields.Str()
    value_choices = fields.List(fields.Nested(ValueChoiceScheme))
    value_error = fields.Str()

    @post_load
    def make_object(self, data):
        return Param(**data)


class Asset(BaseObject):
    pass


class AssetScheme(BaseScheme):
    status = fields.Str()
    external_id = fields.Str()
    external_uid = fields.UUID()
    product = fields.Nested(ProductScheme, only=('id', 'name'))
    connection = fields.Nested(
        ConnectionScheme, only=('id', 'type', 'provider', 'vendor'),
    )
    items = fields.List(fields.Nested(ItemScheme))
    params = fields.List(fields.Nested(ParamsScheme))
    tiers = fields.Nested(TiersSchemeMixin)

    @post_load
    def make_object(self, data):
        return Asset(**data)


class Marketplace(BaseObject):
    pass


class MarketplaceScheme(BaseScheme):
    name = fields.Str()
    zone = fields.Str()
    description = fields.Str()
    active_contract = fields.Int()
    icon = fields.Str()
    owner = fields.Nested(CompanyScheme, only=('id', 'name'))
    hubs = fields.List(fields.Nested(HubsSchemeMixin, only=('id', 'name')))

    @post_load
    def make_object(self, data):
        return Marketplace(**data)


class Agreement(BaseObject):
    pass


class AgreementScheme(BaseScheme):
    type = fields.Str()
    title = fields.Str()
    description = fields.Str()
    created = fields.DateTime()
    updated = fields.DateTime()
    owner = fields.Nested(CompanyScheme, only=('id', 'name'))
    stats = fields.Dict()
    active = fields.Bool()
    version = fields.Int()
    link = fields.Str()
    version_created = fields.DateTime()
    version_contracts = fields.Int()

    @post_load
    def make_object(self, data):
        return Agreement(**data)


class Contract(BaseObject):
    pass


class ContractScheme(BaseScheme):
    name = fields.Str()
    status = fields.Str()
    version = fields.Int()
    type = fields.Str()
    agreement = fields.Nested(AgreementScheme, only=('id', 'name'))
    marketplace = fields.Nested(MarketplaceScheme, only=('id', 'name'))
    owner = fields.Nested(CompanyScheme, only=('id', 'name'))
    creater = fields.Nested(CompanyScheme, only=('id', 'name'))
    created = fields.DateTime()
    updated = fields.DateTime()
    enrolled = fields.Str()
    version_created = fields.DateTime()
    activation = fields.Dict()
    signee = fields.Nested(CompanyScheme, only=('id', 'name'))

    @post_load
    def make_object(self, data):
        return Contract(**data)


class Fulfillment(BaseObject):
    pass


class FulfillmentScheme(BaseScheme):
    activation_key = fields.Str()
    asset = fields.Nested(AssetScheme)
    status = fields.Str()
    type = fields.Str()
    updated = fields.DateTime()
    created = fields.DateTime()
    reason = fields.Str()
    params_form_url = fields.Str()
    contract = fields.Nested(ContractScheme, only=('id', 'name'))
    marketplace = fields.Nested(MarketplaceScheme, only=('id', 'name'))

    @post_load
    def make_object(self, data):
        return Fulfillment(**data)


class ServerError(BaseObject):
    pass


class ServerErrorScheme(Schema):
    error_code = fields.Str()
    params = fields.Dict()
    errors = fields.List(fields.Str())

    @post_load
    def make_object(self, data):
        return ServerError(**data)



from .schema import ServerError


class Message(Exception):
    def __init__(self, message='', code='', obj=None):
        self.message = message
        self.code = code
        self.obj = obj


class FulfillmentFail(Message):
    def __init__(self, *args, **kwargs):
        super(FulfillmentFail, self).__init__(*args, **kwargs)
        self.code = 'fail'
        self.message = self.message or 'Request failed'


class FulfillmentInquire(Message):
    def __init__(self, *args, **kwargs):
        super(FulfillmentInquire, self).__init__(*args, **kwargs)
        self.message = self.message or 'Correct user input required'
        self.params = kwargs.get('params', [])
        self.code = 'inquire'


class Skip(Message):
    def __init__(self, *args, **kwargs):
        self.code = 'skip'


class ServerErrorException(Exception):
    message = 'Server error'

    def __init__(self, error=None, *args, **kwargs):
        if error and isinstance(error, ServerError):
            self.message = str({
                "error_code": error.error_code,
                "params": kwargs.get('params', []),
                "errors": error.errors,
            })

        super(ServerErrorException, self).__init__(
            self.message, *args, **kwargs)



from connect.models import ActivationTemplateResponse, ActivationTileResponse
from .base import BaseResource
from .utils import joinurl


class TemplateResource(BaseResource):
    """ Specific resource.
    One method `render`
    He returns json string with activation tile
    """
    resource = 'templates'

    def render(self, pk, request_id):
        if not all([pk, request_id]):
            raise ValueError('Invalid ids for render temlpate')

        url = joinurl(self._obj_url(pk), 'render')
        response = self.api.get(url, params={'request_id': request_id})

        return ActivationTileResponse(response)

    def get(self, pk):
        return ActivationTemplateResponse(template_id=pk)

    def list(self):
        raise AttributeError('This resource do not have method `list`')



from .activation_response import ActivationTemplateResponse, ActivationTileResponse
from .schema import BaseScheme, FulfillmentScheme, Param, ParamsScheme, ServerErrorScheme

__all__ = [
    'ActivationTemplateResponse',
    'ActivationTileResponse',
    'BaseScheme',
    'FulfillmentScheme',
    'ServerErrorScheme',
    'Param',
    'ParamsScheme',
]



# -*- coding: utf-8 -*-

import json
import logging
import os
from logging.config import dictConfig

with open(os.path.join(os.path.dirname(__file__), 'config.json')) as config_file:
    config = json.load(config_file)

dictConfig(config['logging'])
logger = logging.getLogger()


def function_log(func):
    def decorator(self, *args, **kwargs):
        logger.info('Entering: %s', func.__name__)
        logger.debug('Function params: {} {}'.format(args, kwargs))

        result = func(self, *args, **kwargs)

        logger.debug(
            'Function `{}.{}` return: {}'.format(self.__class__.__name__, func.__name__, result))
        return result

    return decorator



# -*- coding: utf-8 -*-

import json
import os


class Config(object):
    api_url = None
    api_key = None
    products = []

    def __init__(
            self,
            api_url=None,
            api_key=None,
            products=None,
            file=None,
            **kwargs
    ):
        """
        initialization config for public api
        :param api_url: Public api url
        :param api_key: Service user ApiKey
        :param products (optional): Id products
        :param file: Config file path
        :param kwargs:
        """
        if not all([Config.api_key, Config.api_url]):
            self.load(api_url, api_key, products, file, **kwargs)

    def load(self, api_url, api_key, products, file, **kwargs):
        if not any([api_key, api_url, file]):
            return

        if file:
            return self.load_from_file(file)

        self.check_credentials(api_url, api_key, products)
        self._set_attr(api_url, api_key, products)

    @staticmethod
    def _set_attr(api_url, api_key, products=None):
        Config.api_key = api_key

        # URL must end with a character
        Config.api_url = api_url
        if products:
            Config.products = [products] if isinstance(
                products, str) else products

    @staticmethod
    def check_credentials(api_url, api_key, products):
        """
        :param api_url:
        :param api_key:
        :param products:
        :return: True or raise ValueError
        """
        if not all([api_key, api_url]):
            raise ValueError('Please provide your credentials.'
                             'Not set value for `api_key` or `api_url`')

        if products and not isinstance(products, (str, list)):
            raise TypeError('Products can be string or string list')

        return

    def load_from_file(self, file):
        """
        Format file (json): {
            "api_key":
            "api_url":
            "products" (optional):
        }
        :param file (str): Path to the file
        :return: Init configuration parameters.
                Set Config.api_url/.api_key/.products
        """
        if not os.path.exists(file):
            raise IOError('Not file `{}` on directory'.format(file))

        with open(file) as config_file:
            configs = config_file.read()

        try:
            configs = json.loads(configs)
        except Exception as ex:
            raise TypeError('Invalid config file `{}`\n'
                            'ERROR: {}'.format(file, str(ex)))

        (api_url, api_key, products) = (configs.get('apiEndpoint', ''),
                                        configs.get('apiKey', ''),
                                        configs.get('products', ''))

        products = products.encode('utf-8') if not isinstance(products, (str, list)) else products
        api_url = api_url.encode('utf-8') if not isinstance(api_url, str) else api_url
        api_key = api_key.encode('utf-8') if not isinstance(api_key, str) else api_key

        self.check_credentials(api_url, api_key, products)
        self._set_attr(api_url, api_key, products)



# -*- coding: utf-8 -*-

from requests.compat import urljoin


def joinurl(base, url, allow_fragments=True):
    """ Method for the correct formation of the URL """
    if base and isinstance(base, str):
        base += '/' if base[-1] != '/' else ''
    return urljoin(base, url, allow_fragments=True)



import json


class ActivationTileResponse(object):
    tile = 'Activation succeeded'

    def __init__(self, markdown=None, *args, **kwargs):
        if markdown:
            try:
                self.tile = json.loads(markdown)
            except ValueError:
                self.tile = markdown


class ActivationTemplateResponse(object):
    def __init__(self, template_id):
        self.template_id = template_id



#!/usr/bin/env python

from os.path import abspath, dirname, exists, join

from setuptools import find_packages, setup

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements

install_reqs = parse_requirements(
    join(
        dirname(abspath(__file__)),
        'requirements',
        'sdk.txt',
    ), session='None')

here = dirname(abspath(__file__))
with open(join(here, 'VERSION')) as f:
    VERSION = f.read()

packages = find_packages(exclude=['tests*'])

doc = ''
if exists('README.md'):
    doc = open('README.md', 'r').read()

setup(
    name='connect-sdk',
    author='Ingram Micro',
    version=VERSION,
    keywords='sdk connect connect automation',
    packages=packages,
    description='Connect Python SDK',
    long_description=doc,
    long_description_content_type='text/markdown',
    url='https://github.com/ingrammicro/connect-python-sdk',
    license='Apache Software License',
    include_package_data=True,
    install_requires=[str(ir.req) for ir in install_reqs],

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'License :: OSI Approved :: Apache Software License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',

        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
    ],
)



from connect.logger import logger
from connect.models import ActivationTemplateResponse, ActivationTileResponse
from connect.models.exception import FulfillmentFail, FulfillmentInquire, Skip
from .fulfillment import FulfillmentResource


class FulfillmentAutomation(FulfillmentResource):

    def process(self):
        for _ in self.list():
            self.dispatch(_)

    def dispatch(self, request):
        try:
            logger.info('Start request process / ID request - {}'.format(request.id))
            result = self.process_request(request)

            if not result:
                logger.info('Method `process_request` did not return result')
                return

            params = {}
            if isinstance(result, ActivationTileResponse):
                params = {'activation_tile': result.tile}
            elif isinstance(result, ActivationTemplateResponse):
                params = {'template_id': result.template_id}

            self.approve(request.id, params)

        except FulfillmentInquire as inquire:
            self.update_parameters(request.id, inquire.params)
            return self.inquire(request.id)

        except FulfillmentFail as fail:
            return self.fail(request.id, reason=fail.message)

        except Skip as skip:
            return skip.code
        return

    def process_request(self, request):
        raise NotImplementedError(
            'Please implementation `process_request` logic')
